import click

from .package_description import PackageDescription
from .uploader import CodebergUploader


@click.group
@click.pass_context
def codeberg(ctx: click.Context):
    """Commands to interact with the versions posted on codeberg"""
    ctx.ensure_object(dict)
    ctx.obj["uploader"] = CodebergUploader.from_env()


@codeberg.command
@click.pass_context
@click.option(
    "--package", default="results_funfedi_connect", help="The results to display"
)
@click.option("--package_version", default="0.1.3", help="The version to display")
@click.argument("filename")
def delete(ctx: click.Context, package: str, package_version: str, filename: str):
    package_description = PackageDescription(package, package_version)

    ctx.obj["uploader"].delete_file(package_description, filename)


if __name__ == "__main__":
    codeberg()



from pathlib import Path


class DocPaths:
    def __init__(self):
        self.snippets = Path("snippets")
        self.snippets.mkdir(exist_ok=True, parents=True)

    @property
    def codeberg_package_links(self):
        return self.snippets / "packages.md"



from unittest.mock import MagicMock, patch

import pytest

from .package_description import KnownPackages, PackageDescription
from .uploader import BadCodebergCredentialsException, CodebergUploader


def test_from_env(monkeypatch):
    monkeypatch.setenv("CODEBERG_USER", "user")
    monkeypatch.setenv("CODEBERG_TOKEN", "token")

    uploader = CodebergUploader.from_env()

    assert uploader.user == "user"
    assert uploader.token == "token"


def test_from_env_bad_credentials(monkeypatch):
    monkeypatch.setenv("CODEBERG_USER", "")
    monkeypatch.setenv("CODEBERG_TOKEN", "")

    with pytest.raises(BadCodebergCredentialsException):
        CodebergUploader.from_env()


@patch("requests.put")
def test_upload_file(requests_mock, tmp_path):
    uploader = CodebergUploader("user", "token")
    package = PackageDescription(KnownPackages.connect, "1.2.3")
    file = tmp_path / "file.txt"

    with open(file, "w") as f:
        f.write("test")

    uploader.upload_file(package, "file.txt", file)

    requests_mock.assert_called_once()


@patch("requests.delete")
def test_delete_file(requests_mock):
    requests_mock.return_value = MagicMock(status_code=204)
    uploader = CodebergUploader("user", "token")
    package = PackageDescription(KnownPackages.connect, "1.2.3")

    uploader.delete_file(package, "file.txt")

    requests_mock.assert_called_once()



import click

from .formatter import package_download_snippet

from . import DocPaths


@click.command
def doc_cli():
    """Writes the docs snippets"""
    paths = DocPaths()

    with paths.codeberg_package_links.open("w") as f:
        f.write(package_download_snippet())


if __name__ == "__main__":
    doc_cli()



from .formatter import package_download_snippet


def test_package_download_snippet():
    result = package_download_snippet()

    assert (
        result
        == """## Packages to download

* [parsing](https://codeberg.org/funfedidev/-/packages/generic/results_funfedi_parsing)
* [parsing_notes](https://codeberg.org/funfedidev/-/packages/generic/results_funfedi_parsing_notes)
* [parsing_events](https://codeberg.org/funfedidev/-/packages/generic/results_funfedi_parsing_events)
* [connect](https://codeberg.org/funfedidev/-/packages/generic/results_funfedi_connect)"""
    )



from funfedi_results.codeberg import (
    CodebergPackageDownloader,
    KnownPackages,
    PackageDescription,
)


def package_download_snippet():
    lines = ["## Packages to download", ""]

    for package in KnownPackages:
        downloader = CodebergPackageDownloader(PackageDescription(package, "0.0.1"))
        lines.append(f"* [{package.name}]({downloader.download_url})")

    return "\n".join(lines)
